import fnmatch
import os
import argparse
import sys
import re
from ete3 import Tree
import time
from datetime import timedelta, date
import csv
import json
from statistics import mode
import svgwrite #conda install svgwrite
import random
import cairo
import cairosvg



def main():


	MIN_PYTHON = (3, 7)
	if sys.version_info < MIN_PYTHON:
		sys.exit("Python %s.%s or later is required.\n" % MIN_PYTHON)

	print("Loaded imports")

	parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)


	##########################  parse user arguments
	parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

	global_group  = parser.add_argument_group('Options for full repot')

	global_group.add_argument('-oDir', '--outDirectory', required=False, default ="./", type=str, help="folder for output")
	global_group.add_argument('-oP', '--outPrefix', required=True, type=str, help="prefix of out files withen outDirectory")


	defClades_group  = parser.add_argument_group('Options passed to epimuller-define')

	defClades_group.add_argument('-n', '--inNextstrain', required=True, type=str, help="nextstrain results with tree.nwk and [traitOfInterst].json")
	defClades_group.add_argument('-m', '--inMeta', required=True, type=str, help="metadata tsv with 'strain'	and 'date'cols, optional: cols of trait of interst; and pangolin col named: 'lineage' or 'pangolin_lin'")
	defClades_group.add_argument('-p', '--inPangolin', required=False, type=str, default = "metadata", help="pangolin output lineage_report.csv file, if argument not supplied looks in inMeta for col with 'pangolin_lin' or 'lineage'")

	defClades_group.add_argument('-f', '--traitOfInterstFile', required=False, type=str, default="aa_muts.json",  help="name of nextstrain [traitOfInterst].json in 'inNextstrain' folder")
	defClades_group.add_argument('-k', '--traitOfInterstKey', required=False, type=str, default="aa_muts",  help="key for trait of interst in json file")
	defClades_group.add_argument('-aa', '--aaVOClist', required=False, nargs='+', help="list of aa of interest in form [GENE][*ORAncAA][site][*ORtoAA] ex. S*501*, gaps represed by X")


	defClades_group.add_argument('-t', '--timeWindow', required=False, type=str, default="7", help="number of days for sampling window")
	defClades_group.add_argument('-s', '--startDate', required=False, type=str, default='2020-03-01', help="start date in iso format YYYY-MM-DD or 'firstDate' which sets start date to first date in metadata")
	defClades_group.add_argument('-e', '--endDate', required=False, type=str, default='lastDate', help="end date in iso format YYYY-MM-DD or 'lastDate' which sets end date as last date in metadata")

	

	##########################  command line args for drawMuller.py

	drawing_group  = parser.add_argument_group('Options passed to epimuller-draw')

	drawing_group.add_argument('-mt', '--MINTIME', required=False, type=str, default="30", help="minimum time point to start plotting")
	drawing_group.add_argument('-min', '--MINTOTALCOUNT', required=False, type=str, default="10", help="minimum total count for group to be included")

	drawing_group.add_argument('-c', '--cases_name', required=False, type=str, help="file with cases - formated with 'date' in ISO format and 'confirmed_rolling' cases, in tsv format")

	drawing_group.add_argument('-l', '--xlabel', required=False, type=str, choices = ["date", "time"], default="date", help="Format of x axis label: ISO date format or timepoints from start")
	drawing_group.add_argument('-lp', '--labelPosition', required=False, type=str, default="Right", choices = ["Right", "Max", "Start", "End"], help="choose position of clade labels")



	args = parser.parse_args()


	########################### call defineAndCountClades.py



	# oscommand = " ".join(["python 00_scripts/defineAndCountClades.py --outDirectory", args.outDirectory , "--outPrefix", args.outPrefix ,
	#  "--inNextstrain", args.inNextstrain , "--inMeta", args.inMeta , "--inPangolin", args.inPangolin , 
	#  "--traitOfInterstFile", args.traitOfInterstFile , "--traitOfInterstKey", args.traitOfInterstKey , "--timeWindow", args.timeWindow ,
	#  "--startDate", args.startDate ,"--endDate", args.endDate])
	oscommand = " ".join(["epimuller-define --outDirectory", args.outDirectory , "--outPrefix", args.outPrefix ,
	 "--inNextstrain", args.inNextstrain , "--inMeta", args.inMeta , "--inPangolin", args.inPangolin , 
	 "--traitOfInterstFile", args.traitOfInterstFile , "--traitOfInterstKey", args.traitOfInterstKey , "--timeWindow", args.timeWindow ,
	 "--startDate", args.startDate ,"--endDate", args.endDate])
	if args.aaVOClist is not None:
		oscommand = oscommand + " --aaVOClist"
		for aa in args.aaVOClist:
			oscommand += " " + aa


	print("\n ### Call command ###")
	print(oscommand)
	if os.system(oscommand) != 0:
		sys.exit("epimuller-define failed")


	##########################  call drawMuller.py


	outCladeHierarchy_name = os.path.join(args.outDirectory, args.outPrefix +"_defineCountClades", "cladeHierarchy.csv")
	outCounts_name = os.path.join(args.outDirectory, args.outPrefix +"_defineCountClades", "abundances.csv")
	plot_folder = os.path.join(args.outDirectory, args.outPrefix+ "_PLOTS", "")


	# oscommand = " ".join(["python 00_scripts/drawMuller.py --parentHierarchy_name", outCladeHierarchy_name, "--abundance_name", outCounts_name, "--outFolder", plot_folder,
	# 	"--xlabel", args.xlabel, "--labelPosition", args.labelPosition, "--MINTIME", args.MINTIME, "--MINTOTALCOUNT", 
	# 	args.MINTOTALCOUNT, "--xlabel", args.xlabel , "--labelPosition", args.labelPosition])
	oscommand = " ".join(["epimuller-draw --parentHierarchy_name", outCladeHierarchy_name, "--abundance_name", outCounts_name, "--outFolder", plot_folder,
		"--xlabel", args.xlabel, "--labelPosition", args.labelPosition, "--MINTIME", args.MINTIME, "--MINTOTALCOUNT", 
		args.MINTOTALCOUNT, "--xlabel", args.xlabel , "--labelPosition", args.labelPosition])

	if args.cases_name is not None:
		oscommand = oscommand + " --cases_name " + args.cases_name
	
	print("\n ### Call command ###")
	print(oscommand)
	os.system(oscommand)




if __name__ == "__main__":
	MIN_PYTHON = (3, 7)
	if sys.version_info < MIN_PYTHON:
		sys.exit("Python %s.%s or later is required.\n" % MIN_PYTHON)

	print("Loaded imports")
	main()


import fnmatch
import os
import argparse
import sys



def main():
	"""
	Input: fasta with sequence id that has *bar*isodate suffix
	Output: metadat file that can be input into nextstrain (as downloaded 2021-02-01) fasta with *bar* to "_" 
	"""

	# parse user arguments
	parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
	parser.add_argument('-iF', '--inFasta', required=True, type=str, help="full metadata file with ISO date after last '|' in name")
	parser.add_argument('-oM', '--outMeta', required=True, type=str, help="output for metadata file")
	parser.add_argument('-oF', '--outFasta', required=True, type=str, help="output for fasta file")
	parser.add_argument('-p', '--inPangolin', required=False, type=str, default = "metadata", help="pangolin output lineage_report.csv file, if argument not supplied adds ? in 'lineage' col")


	args = parser.parse_args()

	inFasta_open = open(args.inFasta, "r")
	outFasta_open = open(args.outFasta, "w")
	outMeta_open = open(args.outMeta, "w")

	lineage_d = {}
	if args.inPangolin is not None:
		haveLineage = True
		inPangolin_open = open(args.inPangolin, "r")
		for line in inPangolin_open:
			line_l = line.strip().split(",")
			name = line_l[0].replace("|", "_")
			lineage = line_l[1]
			lineage_d[name] = lineage
	else:
		haveLineage = False


	outMetaLine = "strain	virus	date	region	country_exposure	date_submitted	lineage\n"
	outMeta_open.write(outMetaLine)

	inName = 'NA'

	for line in inFasta_open:
		if ">" in line:
			if inName != 'NA' and 'Wuhan' not in inName:
				outFasta_open.write(outFastaLine)
				outFasta_open.write(seq)
				outMeta_open.write(outMetaLine)

			inName = line.strip()[1:]

			date = inName.split("|")[-1]
			outName = inName.replace("|", "_")
			outFastaLine = ">" + outName + "\n"
			if haveLineage:
				outMetaLine = "	".join([outName, "ncov", date, "North America", "?	2030-03-22", lineage_d[outName] +"\n"]) 
			else:
				outMetaLine = "	".join([outName, "ncov", date, "North America", "?	2030-03-22	?\n"]) 
			
			seq = ""
		else:
			seq += line
	outFasta_open.write(outFastaLine)
	outFasta_open.write(seq)
	outMeta_open.write(outMetaLine)



if __name__ == "__main__":
	main()





import fnmatch
import os
import argparse
import sys
import re
from ete3 import Tree
import time
from datetime import timedelta, date
import datetime
import csv
import json
from statistics import mode




def nwkToAnnotNameNWK(t, j_d, trait, inMeta_name, pangolin, indexToGene, geneToIndex):
	'''
	Input: ete3 tree with node names that have 'trait' of clade specified in j_d
	Outputs: tree with trait appened to node names
	'''
	firstDate = date.fromisoformat('2030-01-01')
	lastDate =  date.fromisoformat('1900-01-01')

	sampPangolin_d = {}
	sampDate_d = {}
	strain_index = "NA"
	pangolin_index = "NA"
	inMeta_open = open(inMeta_name, "r")
	for line in inMeta_open:
		line_l = re.split('\t|,', line.strip())
		if strain_index != "NA":
			tipName = line_l[strain_index]
			tipDate = line_l[date_index]

			sampDate_d[tipName] = tipDate 

			if tipDate != "":
				isoDate = date.fromisoformat(tipDate)

				if isoDate < firstDate:
					firstDate = isoDate
				if isoDate > lastDate:
					lastDate = isoDate

			if pangolin == 'metadata':
				sampPangolin = line_l[pangolin_index]
				sampPangolin_d[tipName] = sampPangolin

		elif "strain" in line_l and "date" in line_l:
			strain_index = line_l.index("strain")
			date_index = line_l.index("date")

		elif "Name" in line_l and "Date" in line_l:
			strain_index = line_l.index("Name")
			date_index = line_l.index("Date")

		else:
			sys.exit(line_l)
			#"Must have 'strain' and 'date' in first line of tab seperated"

		if pangolin == 'metadata' and pangolin_index == "NA":
			if "pangolin_lin" in line_l:
				pangolin_index = line_l.index("pangolin_lin")
			if "lineage" in line_l:
				pangolin_index = line_l.index("lineage")
	
	if pangolin != 'metadata': # pangolin output lineage report file name
		inPangolin_open = open(pangolin, "r")
		for line in inPangolin_open:
			line_l = line.split(",")
			if line_l[2] != "probability": #is not first line
				if float(line_l[2]) > 0.9:
					sampPangolin = line_l[1]
				else:
					print("Prob does not meet threshold", tipName, float(line_l[2]) )
					sampPangolin = 'NA'
				tipName = line_l[0]
				sampPangolin_d[tipName] = sampPangolin


	for node in t.traverse("preorder"):
		if "NODE_" not in node.name:
			if node.name in sampPangolin_d:
				pID = sampPangolin_d[node.name]
			else: 
				pID = "NA"
			temp = "_" + pID + "_" + sampDate_d[node.name] + "_"
		else:
			temp = "_"

		if trait == "aa_muts":
			#for gene in geneToIndex:
			for index in range(12):
				gene = indexToGene[index]
				if gene in j_d['nodes'][node.name][trait]:
					for mut in j_d['nodes'][node.name][trait][gene]:
						usable = mut.replace(".", "J").replace("-", "X").replace("*", "J") #gaps represented by X
						temp += usable+"."
				temp += "-"
		else:
			temp += str(j_d['nodes'][node.name][trait])
			#node_annot = temp
		node.name = node.name + temp
	return(t, firstDate, lastDate)


def assignToSpecAA(t, mutList, geneToIndex, logNotes_open):
	'''
	input: annotated tree (t) and aa mut (mutList) to look for
	output:
	assignment_d: key: leaf node name; value: clade
	heierarchy (heiarchy_d: key:child clade; value:parent clade )
	'''
	clade_s = set()
	assignment_d = {}
	heiarchy_d = {}
	cladesAssinged_d ={}

	currentClade = 'anc'
	heiarchy_d['anc'] = 'NA'
	clade_s.add('anc')

	for node in t.traverse("preorder"):
		assignment_d[node.name] = currentClade

	for node in t.traverse("preorder"):

		node_name = node.name

		nodeMut_l = node_name.split("_")[-1].split("-")
		currentClade = assignment_d[node_name]

		mutOfInterst_count = 0

		for gene_and_mutToFind in mutList:
			for gene in geneToIndex:
				if gene == gene_and_mutToFind[:len(gene)]:
					mutToFind = gene_and_mutToFind[len(gene):]

					nodeMut_gene_catString = nodeMut_l[geneToIndex[gene]]
					nodeMut_gene_l = nodeMut_gene_catString.split(".")
					if nodeMut_gene_l != ['']:
						for nodeMut_one in nodeMut_gene_l:
							if nodeMut_one != '':

								if mutToFind[0] == "*" or mutToFind[0] == nodeMut_one[0]:
									if mutToFind[-1] == "*" or mutToFind[-1] == nodeMut_one[-1]:
										if mutToFind[1:-1] == nodeMut_one[1:-1]:
											mutOfInterst_count += 1
											mutFound = nodeMut_one
		if mutOfInterst_count == 1:
			if mutFound not in cladesAssinged_d:
				cladesAssinged_d[mutFound] = 0
			cladesAssinged_d[mutFound] += 1

			newClade = mutFound+ "_" + str(cladesAssinged_d[mutFound])
			clade_s.add(newClade)

			heiarchy_d[newClade] = currentClade
			assignment_d[node_name] = newClade

			for des_node in node.iter_descendants("postorder"):
				assignment_d[des_node.name] = newClade

		if mutOfInterst_count > 1:
			#print("found more than 1 mut of interst", node_name)
			logNotes_open.write("\n"+"found more than 1 mut of interst"+"\n")
			logNotes_open.write(node_name+"\n")



	return(assignment_d, heiarchy_d, clade_s)


def assignToTraits(t, ofInterst_l = []):
	'''
	input: annotated tree (t) 
	Not currntly functional: if ofInterst_l is not empty only initialize when node has trait in least
	output:
	assignment_d: key: leaf node name; value: clade
	heierarchy (heiarchy_d: key:child clade; value:parent clade )
	'''
	clade_s = set()
	assignment_d = {}
	heiarchy_d = {}
	cladesAssinged_d ={}

	currentClade = 'anc'
	heiarchy_d['anc'] = 'NA'
	clade_s.add('anc')

	for node in t.traverse("preorder"):
		assignment_d[node.name] = currentClade

	for node in t.traverse("preorder"):

		if not node.is_root():

			node_name = node.name

			traitOfNode = node_name.split("_")[-1]
			currentClade = assignment_d[node_name]


			#define new calde because does not match parent
			if currentClade.split("_")[0] != traitOfNode:
				if traitOfNode not in cladesAssinged_d:
					cladesAssinged_d[traitOfNode] = 0
				cladesAssinged_d[traitOfNode] += 1
				newClade = traitOfNode + "_" + str(cladesAssinged_d[traitOfNode])
				clade_s.add(newClade)

				heiarchy_d[newClade] = currentClade
				assignment_d[node_name] = newClade

				for des_node in node.iter_descendants("postorder"):
					assignment_d[des_node.name] = newClade


	return(assignment_d, heiarchy_d, clade_s)




def assignCladeToLin(assignment_old_d, heiarchy_old_d, clade_old_s):
	"""
	mode of Pangolin linage of tips withen clade are appended to clade names
	"""

	reassignCladeNames ={}
	clade_s = set()
	assignment_d = {}
	heiarchy_d = {}
	
	for clade in clade_old_s:
		tipLinForClade = []
		for node in assignment_old_d:
			if 'NODE' not in node:
				if assignment_old_d[node] == clade:
					#print(node)
					tipLinForClade = tipLinForClade + [node.split("_")[-3]]
		if clade != 'anc':
			try:
				lineage = mode(tipLinForClade)
			except ValueError:
				lineage = "2lin"
			newClade = lineage + "_" + clade
		else:
			newClade = clade

		
		reassignCladeNames[clade] = newClade
		clade_s.add(newClade)

	for child in heiarchy_old_d:
		parent = heiarchy_old_d[child]
		if parent != 'NA':
			heiarchy_d[reassignCladeNames[child]] = reassignCladeNames[parent]
		else:
			heiarchy_d[reassignCladeNames[child]] = parent
	for samp in assignment_old_d:
		assignment_d[samp] = reassignCladeNames[assignment_old_d[samp]]

	return (assignment_d, heiarchy_d, clade_s)


def countAbudanceFromNames_byWeek(assignment_d, clade_s, startDate, endDate, delta, tipLog_name):
	"""
	counts total number of tips withen each clade, for each time interval (delta) between startDate and endDate 
	"""
	abundances_d = {} # key: week; value: dict of key:clade; value: count
	weekToDate_d = {}
	#assignment_d: key: node name; value: clade

	week_l = []

	tipLog_open = open(tipLog_name, "w")
	outLine = "	".join(["Week", "Clade", "sample_withAnnot", "sample_inputID"]) + "\n"
	tipLog_open.write(outLine)

	currentStart = startDate
	currentEnd = currentStart + delta
	week = 0
	weekName = str(week)
	week_l = week_l + [weekName]
	abundances_d[weekName] = {}

	while endDate >= currentEnd:

		lastWeekName = weekName
		week += 1
		weekName = str(week)
		week_l = week_l + [weekName]

		weekToDate_d[weekName] = currentStart

		abundances_d[weekName] = {} 

		for clade in abundances_d[lastWeekName]:
			abundances_d[weekName][clade] = 1


		for tip in assignment_d.keys():
			if "NODE_" not in tip and "Wuhan" not in tip:
				tip_date = date.fromisoformat(tip.split("_")[-2])

				if tip_date < currentEnd and tip_date >= currentStart:

					clade = assignment_d[tip]
					if clade not in abundances_d[weekName]:
						abundances_d[weekName][clade] = 1
					abundances_d[weekName][clade] += 1
					if clade != 'anc':
						tipNoAnnot = ""
						for name in tip.split("_")[:-3]: 
							tipNoAnnot = tipNoAnnot + "_"  + name
						outLine = "	".join([ weekName, clade, tip, tipNoAnnot[1:]]) + "\n"
						tipLog_open.write(outLine)


		currentEnd += delta
		currentStart += delta
	tipLog_open.close()

	#print(list(clade_s))
	noFurtherAbudance = list(clade_s.copy())
	for weekName in (reversed(week_l)):

		noFurtherAbudance_last = noFurtherAbudance.copy()
		for clade in noFurtherAbudance_last:
			if clade in abundances_d[weekName]:

				if abundances_d[weekName][clade] == 1 and clade != 'anc':
					abundances_d[weekName][clade] = 0
				else:
					noFurtherAbudance.remove(clade)

	return(abundances_d, weekToDate_d)


def main():



	##########################  parse user arguments
	parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

	parser.add_argument('-n', '--inNextstrain', required=True, type=str, help="nextstrain results with tree.nwk and [traitOfInterst].json")
	parser.add_argument('-m', '--inMeta', required=True, type=str, help="metadata tsv with 'strain'	and 'date'cols, optional: cols of trait of interst; and pangolin col named: 'lineage' or 'pangolin_lin'")
	parser.add_argument('-p', '--inPangolin', required=False, type=str, default = "metadata", help="pangolin output lineage_report.csv file, if argument not supplied looks in inMeta for col with 'pangolin_lin' or 'lineage'")

	parser.add_argument('-f', '--traitOfInterstFile', required=False, type=str, default="aa_muts.json",  help="name of nextstrain [traitOfInterst].json in 'inNextstrain' folder")
	parser.add_argument('-k', '--traitOfInterstKey', required=False, type=str, default="aa_muts",  help="key for trait of interst in json file")
	parser.add_argument('-aa', '--aaVOClist', required=False, nargs='+', help="list of aa of interest in form [GENE][*ORAncAA][site][*ORtoAA] ex. S*501*, gaps represed by X")



	parser.add_argument('-oDir', '--outDirectory', required=False, default ="./", type=str, help="folder for output")
	parser.add_argument('-oP', '--outPrefix', required=True, type=str, help="prefix of out files withen outDirectory")


	parser.add_argument('-t', '--timeWindow', required=False, type=str, default="7", help="number of days for sampling window")
	parser.add_argument('-s', '--startDate', required=False, type=str, default='2020-03-01', help="start date in iso format YYYY-MM-DD or 'firstDate' which is in metadata")
	parser.add_argument('-e', '--endDate', required=False, type=str, default='lastDate', help="end date in iso format YYYY-MM-DD or 'lastDate' which is in metadata")

	#parser.add_argument('-w', '--MINTIME', required=False, type=str, default="30", help="minimum time point to start plotting")
	#parser.add_argument('-min', '--MINTOTALCOUNT', required=False, type=str, default="10", help="minimum total count for group to be included")

	

	args = parser.parse_args()

	##########################  file names

	inTree_name = os.path.join(args.inNextstrain, "tree.nwk")
	
	
	inJSON_name = os.path.join(args.inNextstrain, args.traitOfInterstFile)
	inMeta_name = args.inMeta
	# if args.inPangolin is None:
	# 	pangolin = 'metadata'
	# else:
	pangolin = args.inPangolin
	
	outCladeHierarchy_name = os.path.join(args.outDirectory, args.outPrefix +"_defineCountClades", "cladeHierarchy.csv")
	outCounts_name = os.path.join(args.outDirectory, args.outPrefix +"_defineCountClades", "abundances.csv")
	tipLog_name = os.path.join(args.outDirectory, args.outPrefix +"_defineCountClades", "tipsWithMut.txt")

	logNotes_name = os.path.join(args.outDirectory, args.outPrefix + "_defineCountClades", "log.txt")


	# weekTreeFile_folder = os.path.join(args.outDirectory, args.outPrefix+ "_PrunedTrees")
	# weekTreeFile_prefix = os.path.join(weekTreeFile_folder, "PrunedWeek_")



	if not os.path.exists(args.outDirectory):
		os.makedirs(args.outDirectory)
	# if not os.path.exists(weekTreeFile_folder):
	#  	os.makedirs(weekTreeFile_folder)
	dataOutDir = os.path.join(args.outDirectory, args.outPrefix + "_defineCountClades")
	if not os.path.exists(dataOutDir):
	 	os.makedirs(dataOutDir)


	######################### params to referance

	geneToIndex = {"E":0, "M":1, "N":2, "ORF1a":3, "ORF1b":4, "ORF3a":5, "ORF6":6, "ORF7a":7, "ORF7b":8, "ORF8":9, "ORF9b":10, "S":11}
	indexToGene = {0:"E", 1:"M", 2:"N", 3:"ORF1a", 4:"ORF1b", 5:"ORF3a", 6:"ORF6", 7:"ORF7a", 8:"ORF7b", 9:"ORF8", 10:"ORF9b", 11:"S"}


	delta= timedelta(days=int(args.timeWindow))


	#########################  load nwk files

	print(inTree_name)
	if not os.path.exists(inTree_name):
		sys.exit("missing input tree")
	t = Tree(inTree_name, format = 3)


	j_d = json.load(open(inJSON_name))

	
	t, firstDate, lastDate = nwkToAnnotNameNWK(t, j_d, args.traitOfInterstKey, inMeta_name, pangolin, indexToGene, geneToIndex)

	if args.startDate == "firstDate":
		startDate = firstDate
	else:
		startDate =  date.fromisoformat(args.startDate)
	if args.endDate == "lastDate":
		endDate = lastDate
	else:
		endDate =  date.fromisoformat(args.endDate)	



	######################### make clades assignments (assignment_d: key: leaf node name; value: clade) and heierarchy (heiarchy_d: key:child clade; value:parent clade )

	logNotes_open = open(logNotes_name, "w")
	if args.traitOfInterstKey == 'aa_muts':
		if args.aaVOClist is None:
			#aa_mut = ['S*501*', 'ORF1b*1183*', 'S*484*', 'S*452*', 'S*681*', 'S*13', 'S*152']
			#aa_mut = ['S*484*', 'S*501*'] #default sites of interst
			aa_mut = ["S*484*", "S*501*", "S*13*", "S*452*", "S*477*", "N*205*", "S*253*"]
			# aa_mut = ["S*484*", "S*501*", "S*13*", "S*452*", "S*477*", "S*253*"]
			# aa_mut = ['S*484*']

		else:
			aa_mut = args.aaVOClist

		print("\n")
		print("Searching for muations with:", aa_mut)

		
		logNotes_open.write("\n"+"Searching for muations with:"+str(aa_mut)+"\n")


		assignment_d, heiarchy_d, clade_s = assignToSpecAA(t, aa_mut, geneToIndex, logNotes_open)
	else:
		logNotes_open.write("\n"+"Finding clades defined by"+str(args.traitOfInterstKey)+"\n")
		assignment_d, heiarchy_d, clade_s = assignToTraits(t)

	assignment_d, heiarchy_d, clade_s = assignCladeToLin(assignment_d, heiarchy_d, clade_s)


	######################### count abundances based on assignment_d


	abundances_d, weekToDate_d = countAbudanceFromNames_byWeek(assignment_d, clade_s, startDate, endDate, delta, tipLog_name)

	#abundances_d, heiarchy_d = clipPreParent(abundances_d, heiarchy_d)


	# ##########################  write abundances over time

	outCounts_open = open(outCounts_name, "w")
	headLine = "names,times,abundances,date\n"
	outCounts_open.write(headLine)

	for week in abundances_d:
		for name in abundances_d[week]:
			outLine = ",".join([name, str(week.replace("week", "")), str(abundances_d[week][name]), str(weekToDate_d[week])]) + "\n"
			outCounts_open.write(outLine)
	outCounts_open.close()

	##########################  write CladeHierarchy to file

	outCladeHierarchy_open = open(outCladeHierarchy_name, "w")
	headLine = "names,parents \n"
	outCladeHierarchy_open.write(headLine)
	for child in heiarchy_d:
		outLine = child + "," + heiarchy_d[child] + "\n"
		outCladeHierarchy_open.write(outLine)

	outCladeHierarchy_open.close()


	#print("FINAL heiarchy_d")
	#print(heiarchy_d)
	logNotes_open.write("\n"+"Final heiarchy_d"+"\n")
	logNotes_open.write(str(heiarchy_d)+"\n")


if __name__ == "__main__":
	MIN_PYTHON = (3, 7)
	if sys.version_info < MIN_PYTHON:
		sys.exit("Python %s.%s or later is required.\n" % MIN_PYTHON)

	print("Loaded imports")
	main()


import fnmatch
import os
import argparse
import sys
import math
import svgwrite #conda install svgwrite
import random
import cairo
import cairosvg


WIDTH = 1500
HEIGHT = 1000
#LEGENDWIDTH_GLOBAL = 220
LEGENDWIDTH = 220
MARGIN = 60
LABELSHIFT = 15
FONTSIZE = 26

class Clade:
	def __init__(self, name, parent_name):
		self.name = name
		self.parent_name = parent_name
		self.cladeSnapshot_time_d = {} 
		self.y1_d = {}
		self.y2_d = {}

class Snapshot:
	def __init__(self, time, date):
		self.time = time
		self.date = date
		self.label = "t_" + str(time)
		self.cladeSnapshot_clade_d = {}
		self.sumAll = 0 


class CladeSnapshot:
	def __init__(self, clade, snapshot, abundance):
		self.clade = clade
		self.snapshot = snapshot
		self.abundance = abundance
		self.sumDescendant = 0 
		#self.numChildren =  len(self.clade.children)
		#self.numChildren = 0
		#to be added later sumDescendant , numChildren

	def sumUpDescendants(self):
		summed = 0
		for child in self.clade.children:
			child_tSnap = child.cladeSnapshot_time_d[self.snapshot.time]
			if child.children_count > 0:
				summed += child_tSnap.sumUpDescendants()
			summed += child_tSnap.abundance
		return summed


def makeColor():
	# random_number = random. randint(0,16777215)
	# color = "#" + str(hex(random_number))[2:]
	color = "#{:06x}".format(random.randint(0, 0xFFFFFF))
	return color

def timeToX(time, scaleTime, minTime):
	return(scaleTime*(int(time) - int(minTime)) + MARGIN)



def textwidth(text, fontsize):
	#function copied from http://blog.mathieu-leplatre.info/text-extents-with-python-cairo.html
	try:
		import cairo
	except Exception as e:
		return len(str) * fontsize
	surface = cairo.SVGSurface('undefined.svg', 1280, 200)
	cr = cairo.Context(surface)
	cr.select_font_face('Arial', cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
	cr.set_font_size(fontsize)
	xbearing, ybearing, width, height, xadvance, yadvance = cr.text_extents(text)
	return width


def textheight(text, fontsize):
	#function based on textwidth copied from http://blog.mathieu-leplatre.info/text-extents-with-python-cairo.html
	try:
		import cairo
	except Exception as e:
		return len(str) * fontsize
	surface = cairo.SVGSurface('undefined.svg', 1280, 200)
	cr = cairo.Context(surface)
	cr.select_font_face('Arial', cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
	cr.set_font_size(fontsize)
	xbearing, ybearing, width, height, xadvance, yadvance = cr.text_extents(text)
	return height

# def makeNoScaleCord(abundances_d, childParent_d, timeToDate_d):
# 	'''
# 	returns list of shape_l where shape_l is a list cord_l, which is cordinates for a clade, fomated as [(x, y), (x, y), ...] relative to top left
# 	'''
# 	shape_l	= []
# 	return shape_l	

def drawWrapper(outFolder, outPrefix, root_clades_l, scaleTime, times_l, maxY, minTime, labelPosition, xlabel, timeToDate_d):

	#Draw background

	outFile =  os.path.join(outFolder, outPrefix + ".svg")
	outFilePDF =  os.path.join(outFolder, outPrefix + ".pdf")

	img = svgwrite.Drawing(filename = outFile, size = (str(WIDTH)+"px", str(HEIGHT)+"px"))
	img.add(img.polyline(points = [(0,0), (0, HEIGHT), (WIDTH, HEIGHT), (WIDTH, 0)], stroke='black', fill = 'white'))
	#img.add(img.text(text = 'Legend', insert = (WIDTH-LEGENDWIDTH, LEGENDWIDTH), font_size=24))
	#rightWidth = (WIDTH-(MARGIN + LEGENDWIDTH + 1/scaleTime)) #WIDTH - (MARGIN + LEGENDWIDTH) 
	rightWidth = timeToX(list(times_l)[-1], scaleTime, minTime)
	img.add(img.polyline(points = [(MARGIN,MARGIN), (MARGIN, HEIGHT-MARGIN), (rightWidth, HEIGHT-MARGIN), (rightWidth, MARGIN), (MARGIN,MARGIN)], stroke='black', stroke_width=5, fill="white"))


	#draw clades
	img, x_labelCord_l, y_labelCord_l, label_l = extractCord_draw(root_clades_l, img, scaleTime, [], [], [], times_l, minTime, labelPosition )


	tHeight = textheight("TESTLABEL", FONTSIZE)
	#write clade labels
	if labelPosition == "Right":
		xpos = timeToX(list(times_l)[-1], scaleTime, minTime)+LABELSHIFT

		zipped = zip(label_l, y_labelCord_l)
		sort_zip = list(sorted(zipped, key = lambda x: x[1]))

		tWidth = textwidth(sort_zip[0][0], FONTSIZE)
		fontsize = FONTSIZE 
		while tWidth > LEGENDWIDTH:
			fontsize += (-1)
			tWidth = textwidth(sort_zip[0][0], fontsize)

		ypos = sort_zip[0][1]
		img.add(img.text(text = sort_zip[0][0], insert = (xpos, ypos), font_size=fontsize))
		for i in range(1, len(sort_zip)):
			bottomPrevious = ypos + LABELSHIFT + tHeight
			ypos = sort_zip[i][1]
			if ypos < bottomPrevious:
				ypos = bottomPrevious

			tWidth = textwidth(sort_zip[i][0], FONTSIZE)
			fontsize = FONTSIZE 
			while tWidth > (LEGENDWIDTH+MARGIN):
				fontsize += (-1)
				tWidth = textwidth(sort_zip[i][0], fontsize)
			
			img.add(img.text(text = sort_zip[i][0], insert = (xpos, ypos), font_size=fontsize))
	else: #if args.labelPosition == "Start":
		for i in range(len(label_l)):
			img.add(img.text(text = label_l[i], insert = (x_labelCord_l[i], y_labelCord_l[i]), font_size=FONTSIZE))


	#wirte y axis labels
	topPlot = MARGIN
	bottomPlot = HEIGHT-MARGIN
	totalHeight = bottomPlot-topPlot
	wirteEvery = int(maxY/10)
	tWidth = textwidth(str(int(maxY)), FONTSIZE)
	fontsize = FONTSIZE 
	while tWidth > MARGIN:
		fontsize = fontsize - 1 
		tWidth = textwidth(str(int(maxY)), fontsize)
	for i in range(int(maxY)):
		if i % wirteEvery == 0:
			xpos = 0
			ypos = MARGIN + (maxY - i)*totalHeight/maxY
			img.add(img.text(text = str(i), insert = (xpos,  ypos), font_size=fontsize))



	#write x axis labels
	if xlabel == "date":
		tWidth = textwidth("2021-05-03", FONTSIZE)
	else:
		tWidth = textwidth("200", FONTSIZE)
	numLabels = rightWidth/tWidth 
	wirteEvery = math.ceil(len(times_l)/numLabels)
	for time in times_l:
		if int(time) % wirteEvery == 0:
			#if int(time) % int(args.XLABFREQ) == 0:
			if xlabel == "time":
				label = str(time)
			else:
				label = timeToDate_d[time]
			img.add(img.text(text = label, insert = (timeToX(time, scaleTime, minTime),  HEIGHT-MARGIN+(2*LABELSHIFT)), font_size=FONTSIZE))
			img.add(img.line(start = (timeToX(time, scaleTime, minTime),  HEIGHT-MARGIN+LABELSHIFT), end = (timeToX(time, scaleTime, minTime),  HEIGHT-(MARGIN+LABELSHIFT)), stroke_width=5, stroke = "black"))
	
	# write title
	fontsize = FONTSIZE+6
	tHeight = textheight(outPrefix, fontsize)
	while tHeight/2 > MARGIN:
		fontsize = fontsize - 1 
		tHeight = textwidth(outPrefix, fontsize)

	img.add(img.text(text = outPrefix, insert = (MARGIN + LABELSHIFT, MARGIN/2), font_size=fontsize))

	img.save()

	cairosvg.svg2pdf(file_obj=open(outFile, "rb"), write_to=outFilePDF)

	




def extractCord_draw(clades_l, img, scaleTime, x_labelCord_l, y_labelCord_l, label_l, times_l, minTime, labelPosition ):
	#for snap in allTimes_l:
	clade_cord_d = {} #key clade name, value is list of coordinate tuples [... (x2, y2t2), (x1, y2t1), (x1, y1t1), (x2, y1t2) ...]
	#clade_col_d = {} #key clade name, value is color
	for clade in clades_l:
		coordinate_l = []
		startAbundance_time = 'NA'
		lastAbundance_time = 0
		maxAbundance_time = 'NA'
		maxAbundance_value = -1
		cladeDrawn = False
		for time in times_l:
			#x = int(time)*scaleTime
			if time in clade.y1_d:
				y1 = clade.y1_d[time]
				y2 = clade.y2_d[time]
				coordinate_l = [(timeToX(time, scaleTime, minTime), y1)] + coordinate_l +  [(timeToX(time, scaleTime, minTime), y2)]
				abundance = clade.cladeSnapshot_time_d[time].abundance/(1.0*clade.cladeSnapshot_time_d[time].snapshot.sumAll)

				if abundance > 0:
					cladeDrawn = True
					if startAbundance_time == 'NA':
						startAbundance_time = time
						startAbundance_y = (y2-y1)//2 + y1
					lastAbundance_time = time
					lastAbundance_y = y1
					drawAbundance = y2-y1
					if maxAbundance_value < drawAbundance: #abundance:
						maxAbundance_value = drawAbundance
						maxAbundance_time = time
						maxAbundance_y = drawAbundance//2 + y1

		clade_cord_d[clade.name] = coordinate_l

		if cladeDrawn:
			if labelPosition == "Right":
				x_cord = timeToX(list(times_l)[-1], scaleTime, minTime)+LABELSHIFT
				y1_cord = y1+LABELSHIFT
			elif labelPosition == "Max":
				x_cord = timeToX(maxAbundance_time, scaleTime, minTime) - textwidth(clade.name, FONTSIZE)//2
				if x_cord < 0:
					x_cord = 0
				y1_cord = maxAbundance_y
			elif labelPosition == "Start":
				x_cord = timeToX(startAbundance_time, scaleTime, minTime)-(textwidth(clade.name, FONTSIZE))
				y1_cord = startAbundance_y
				if x_cord < 0:
					x_cord = 0
			elif labelPosition == "End":
				x_cord = timeToX(lastAbundance_time, scaleTime, minTime)+LABELSHIFT
				if x_cord < 0:
					x_cord = 0
				y1_cord = lastAbundance_y
			else:
				print("do not know where to put clade label")
				sys.exit(1)
			x_labelCord_l.append(x_cord)
			y_labelCord_l.append(y1_cord)


			label_l.append(clade.name)

		
		img.add(img.polyline(points = coordinate_l, stroke='black', stroke_width=0, fill=clade.color))

		img, x_labelCord_l, y_labelCord_l, label_l = extractCord_draw(clade.children, img, scaleTime, x_labelCord_l, y_labelCord_l, label_l, times_l, minTime, labelPosition )


	return(img, x_labelCord_l, y_labelCord_l, label_l)

def removeSmallClades(abundances_d, heiarchy_d, minCount):
	'''
	
	removes clade from abudances and heiarchiy that have sum less than minCount
	heiarchy_d: key:child clade; value:parent clade
	abundances_d: key: week; value: dict of key:clade; value: count
	'''

	# remove clades with less than minCount 

	avalibleClades_d = {}
	avalibleClades_d['anc'] = 0
	avalibleClades_d['NA'] = 0

	avalibleClades_s = set()
	removedClades_s = set()

	for week in abundances_d:
		for clade in abundances_d[week]:
			if clade not in avalibleClades_d:
				avalibleClades_d[clade] = 0
			avalibleClades_d[clade] += int(abundances_d[week][clade])

	old_heiarchy_d = heiarchy_d.copy()
	for clade in old_heiarchy_d: #old_heiarchy_d.keys():
		if clade not in avalibleClades_d:
			heiarchy_d.pop(clade)
			removedClades_s.add(clade)
		elif avalibleClades_d[clade] < int(minCount) and clade != 'NA' and clade != 'anc':
			heiarchy_d.pop(clade)
			removedClades_s.add(clade)
		else:
			avalibleClades_s.add(clade)

	for child in heiarchy_d:
		parent = old_heiarchy_d[child]
		while parent not in heiarchy_d and parent != 'NA':
			newParent = old_heiarchy_d[parent] #heiarchy_d[parent]
			parent = newParent
		heiarchy_d[child] = parent

	# remove from abundances if remved from hieracrhy

	old_abundances_d = abundances_d.copy()
	for week in list(old_abundances_d):
		for clade in list(old_abundances_d[week]):
			if clade in removedClades_s:
				abundances_d[week].pop(clade)

	return(abundances_d, heiarchy_d)


def defineChildBoundries(time, scaleFactor, parentCladeSnap, y1_parent, y2_parent):
	'''
	time is the number defining the time of interest
	parentCladeSnap is pointer to snapshotClade object which has boundries of"
	y1_parent is top of clade boundries, y1_parent is bottom of clade boundries

	updates 
	'''
	numSections = len(parentCladeSnap.clade.children) + 1
	#descendantSpace = (parentCladeSnap.sumDescendant - parentCladeSnap.abundance)
	#parentEdgeHeight = scaleFactor*(parentCladeSnap.abundance - descendantSpace)/numSections	
	parentEdgeHeight = scaleFactor*(parentCladeSnap.abundance)/numSections

	acountedHeight = y1_parent
	for childClade in parentCladeSnap.clade.children:
		childCladeSnap = childClade.cladeSnapshot_time_d[time]
		y1 =  acountedHeight + parentEdgeHeight
		y2 = y1 + (scaleFactor*childCladeSnap.sumDescendant)
		acountedHeight += parentEdgeHeight + (scaleFactor*childCladeSnap.sumDescendant)
		childClade.y1_d[time] = y1
		childClade.y2_d[time] = y2
		defineChildBoundries(time, scaleFactor, childCladeSnap, y1, y2)


def main():
##########################  parse user arguments
	parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)



	parser.add_argument('-p', '--parentHierarchy_name', required=True, type=str, help="csv output from mutationLinages_report.py with child parent col")

	parser.add_argument('-a', '--abundance_name', required=True, type=str, help="csv output from mutationLinages_report.py with abundances of clades")

	parser.add_argument('-c', '--cases_name', required=False, type=str, help="file with cases - formated with 'date' in ISO format and 'confirmed_rolling' cases, in tsv format")

	parser.add_argument('-o', '--outFolder', required=True, type=str, help="csv output from mutationLinages_report.py with child parent col") 


	parser.add_argument('-mt', '--MINTIME', required=False, type=str, default="30", help="minimum time point to start plotting")
	parser.add_argument('-min', '--MINTOTALCOUNT', required=False, type=str, default="10", help="minimum total count for group to be included")

	parser.add_argument('-l', '--xlabel', required=False, type=str, choices = ["date", "time"], default="date", help="Format of x axis label: ISO date format or timepoints from start")
	parser.add_argument('-lp', '--labelPosition', required=False, type=str, default="Right", choices = ["Right", "Max", "Start", "End"], help="choose position of clade labels")

	args = parser.parse_args()


########################## names

	abundances_file = open(args.abundance_name, "r")

	hierarchy_file = open(args.parentHierarchy_name, "r")	

	#color_file = open(args.color_list, "r")

	if not os.path.exists(args.outFolder):
		os.makedirs(args.outFolder)

	# if args.labelPosition != "Right":
	# 	LEGENDWIDTH = 0
	# else:
	# 	LEGENDWIDTH = LEGENDWIDTH_GLOBAL


########################## read in files

	abundances_d = {}
	#abundance_total_d {}
	timeToDate_d = {}
	times_index = 'NA'
	for line in abundances_file:
		line_l = line.strip().split(",")
		if line_l[0] == "names":
			times_index = line_l.index("times")
			a_index = line_l.index("abundances")
			date_index = line_l.index("date")
		elif times_index == 'NA':
			print("First line of abundances_file must start with 'names' col and contain 'times', 'abundances', and 'date' cols, with no spaces between commas\n")
			sys.exit(1)
		else:
			time = line_l[times_index]
			if int(time) >= int(args.MINTIME):
				clade = line_l[0]
				abundance = line_l[a_index]
				if time not in abundances_d:
					abundances_d[time] = {}
				#if clade not in abundance_total_d:
				#	abundance_total_d[clade] = 0
				#abudance_total_d[clade] += abundance
				abundances_d[time][clade] = abundance
				if time not in timeToDate_d:
					timeToDate_d[time] = line_l[date_index]

	abundances_file.close()	

	childParent_d = {}
	cladeColor_d = {}
	hasHeader = False
	hasColor = False
	for line in hierarchy_file:
		line_l = line.strip().split(",")
		if line_l[0] == "names" and line_l[1] == "parents":
			hasHeader = True
			if len(line_l) > 2:
				if line_l[2] == "color":
					hasColor = True
		elif not hasHeader:
			print("First line of parentHierarchy_name must have 'names' as first col and 'parents' as second col, sperated with commas and no spaces")
			sys.exit(1)
		else:
			childParent_d[line_l[0]] = line_l[1]
			if hasColor:
				if line_l[2] != "":
					cladeColor_d[line_l[0]] = line_l[2]
				else:
					cladeColor_d[line_l[0]] = makeColor()
			else:
				cladeColor_d[line_l[0]] = makeColor()

	hierarchy_file.close()

	abundances_d, heiarchy_d = removeSmallClades(abundances_d, childParent_d, args.MINTOTALCOUNT)

	# color_list = []
	# for line in color_file:
	# 	col = line.strip()
	# 	if len(col) == 7:
	# 		color_list.append(col)
	# color_file.close()

########################## parse clades
	times_l = abundances_d.keys()
	#timeLabs_s = set()
	clades_l = []
	root_clades_l = []

	for clade in childParent_d:
		clade = Clade(clade, childParent_d[clade])
		setattr(clade, "color", cladeColor_d[clade.name])
		#setattr(clade, "color", random.choice(color_list))
		#TODO make this determatistic for most diverse colors and add in more color options
		clades_l.append(clade)
		if clade.parent_name == "NA":
			root_clades_l.append(clade)


	#add pointers to children clade objects
	for parent in clades_l:
		setattr(parent, "children", [])
		for child in clades_l:
			if child.parent_name == parent.name:
				parent.children = parent.children + [child]
		setattr(parent, "children_count", len(parent.children))

	#add pointers to parent clade objects
	for child in clades_l:
		setattr(child, "parent", Clade("NA", "NA"))
		for parent in clades_l:
			if parent.name == child.parent_name:
				setattr(child, "parent", parent)
			

	#record abundances in objects
	allTimes_l = []
	for time in abundances_d:
		if time >= args.MINTIME:
			t = Snapshot(time, timeToDate_d[time])
			for clade in clades_l:
				if clade.name in abundances_d[time]:
					abundance = int(abundances_d[time][clade.name])
				else:
					abundance = 0
				clade_oneTime = CladeSnapshot(clade, t, abundance)
				t.cladeSnapshot_clade_d[clade.name] = clade_oneTime
				t.sumAll += abundance
				clade.cladeSnapshot_time_d[time] = clade_oneTime
			allTimes_l.append(t)

	#calculate Descendants and children
	for snap in allTimes_l:
		for cladeSnap in snap.cladeSnapshot_clade_d.values():
			cladeSnap.sumDescendant = cladeSnap.abundance + cladeSnap.sumUpDescendants()

# ################################ determain plotting values

	
	topPlot = MARGIN
	bottomPlot = HEIGHT-MARGIN

	for snap in allTimes_l:
		totalHeight = (bottomPlot-topPlot)
		scaleFactor = totalHeight/snap.sumAll

		descendantSpace = 0
		for clade in root_clades_l:
			descendantSpace += clade.cladeSnapshot_time_d[snap.time].sumDescendant

		numSections = len(root_clades_l) + 1
		parentEdgeHeight = scaleFactor*(snap.sumAll - descendantSpace)/numSections
		#parentEdgeHeight = scaleFactor*(snap.sumAll)/numSections
		acountedHeight = topPlot

		#print(root_clades_l)

		for clade in root_clades_l:
			cladeSnap = clade.cladeSnapshot_time_d[snap.time]
			y1 = acountedHeight + parentEdgeHeight
			y2 = y1 + (scaleFactor*cladeSnap.sumDescendant)
			acountedHeight += (scaleFactor*cladeSnap.sumDescendant) + parentEdgeHeight
			clade.y1_d[snap.time] = y1
			clade.y2_d[snap.time] = y2
			defineChildBoundries(snap.time, scaleFactor, cladeSnap, y1, y2)


	scaleTime = (WIDTH-(MARGIN+LEGENDWIDTH))/len(times_l)
	drawWrapper(args.outFolder, "relative_abundance", root_clades_l, scaleTime, times_l, 100, args.MINTIME, args.labelPosition, args.xlabel, timeToDate_d)



# ########################## make fig with number of samples scaling 

	
	topPlot = MARGIN
	bottomPlot = HEIGHT-MARGIN

	maxCount = 0
	for snap in allTimes_l:
		if snap.sumAll > maxCount:
			maxCount = snap.sumAll


	for snap in allTimes_l:
		#adjust relative to scalling
		lessThanMax = maxCount - snap.sumAll
		lessThanMax_ratio = snap.sumAll/maxCount
		totalHeight = (bottomPlot-topPlot)*lessThanMax_ratio
		
		scaleFactor = totalHeight/snap.sumAll #(maxCount - lessThanMax) #totalHeight/snap.sumAll

		descendantSpace = 0
		for clade in root_clades_l:
			descendantSpace += clade.cladeSnapshot_time_d[snap.time].sumDescendant

		numSections = len(root_clades_l) + 1
		parentEdgeHeight = scaleFactor*(snap.sumAll - descendantSpace)/numSections
		#parentEdgeHeight = scaleFactor*(snap.sumAll)/numSections
		acountedHeight = topPlot + scaleFactor*lessThanMax

		for clade in root_clades_l:


			# should be good

			cladeSnap = clade.cladeSnapshot_time_d[snap.time]
			y1 = acountedHeight + parentEdgeHeight
			y2 = y1 + (scaleFactor*cladeSnap.sumDescendant)
			acountedHeight += (scaleFactor*cladeSnap.sumDescendant) + parentEdgeHeight
			clade.y1_d[snap.time] = y1
			clade.y2_d[snap.time] = y2
			defineChildBoundries(snap.time, scaleFactor, cladeSnap, y1, y2)


	scaleTime = (WIDTH-(MARGIN+LEGENDWIDTH))/len(times_l)
	drawWrapper(args.outFolder, "sequence_scaled_lineages", root_clades_l, scaleTime, times_l, maxCount,   args.MINTIME, args.labelPosition, args.xlabel, timeToDate_d)



########################## make fig with cases scaling
	if args.cases_name is not None:

		if "strain" in line_l and "date" in line_l:
			strain_index = line_l.index("strain")
			date_index = line_l.index("date")

		date_index = "na"
		case_index = "na"
		dateToCase_d = {}
		cases_file = open(args.cases_name, "r")
		for line in cases_file:
			line_l = line.strip().split("	")
			if date_index == "na":
				if "confirmed_rolling" in line_l and "date" in line_l:
					date_index = line_l.index("date")
					case_index = line_l.index("confirmed_rolling")
					#print(line_l, line_l[date_index], line_l[case_index])
				else:
					print("file with cases - formated with 'date' in ISO format and 'confirmed_rolling' cases, in tsv format")
					sys.exit(1)
			else:
				#print(line_l, case_index, date_index , line_l[date_index], line_l[case_index])
				dateToCase_d[line_l[date_index].replace('"', "")] = float(line_l[case_index].replace('"', ""))


		topPlot = MARGIN
		bottomPlot = HEIGHT-MARGIN

		maxCount = 0
		for snap in allTimes_l:
			rollCase = dateToCase_d[timeToDate_d[snap.time]]
			if rollCase > maxCount:
				maxCount = rollCase

		for snap in allTimes_l:

			rollCase = dateToCase_d[timeToDate_d[snap.time]]
			cases_lessThanMax = maxCount - rollCase
			psudoAbundance_lessThanMax = (cases_lessThanMax/rollCase)*snap.sumAll
			lessThanMax_ratio = rollCase/maxCount
			totalHeight = (bottomPlot-topPlot)*lessThanMax_ratio

			
			scaleFactor = totalHeight/snap.sumAll #(maxCount - lessThanMax) #totalHeight/snap.sumAll


			descendantSpace = 0
			for clade in root_clades_l:
				descendantSpace += clade.cladeSnapshot_time_d[snap.time].sumDescendant

			numSections = len(root_clades_l) + 1
			parentEdgeHeight = scaleFactor*(snap.sumAll - descendantSpace)/numSections
			acountedHeight = topPlot + psudoAbundance_lessThanMax*scaleFactor


			for clade in root_clades_l:

				cladeSnap = clade.cladeSnapshot_time_d[snap.time]
				y1 = acountedHeight + parentEdgeHeight
				y2 = y1 + (scaleFactor*cladeSnap.sumDescendant)
				acountedHeight += (scaleFactor*cladeSnap.sumDescendant) + parentEdgeHeight
				clade.y1_d[snap.time] = y1
				clade.y2_d[snap.time] = y2
				defineChildBoundries(snap.time, scaleFactor, cladeSnap, y1, y2)


		scaleTime = (WIDTH-(MARGIN+LEGENDWIDTH))/len(times_l)
		drawWrapper(args.outFolder, "case_scaled_lineages", root_clades_l, scaleTime, times_l, maxCount, args.MINTIME, args.labelPosition, args.xlabel, timeToDate_d)
	else:
		print("No case data supplied - skipping case scaled plot")



if __name__ == "__main__":
	main()


#!/usr/bin/env python
from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

version='0.0.5'

requirements = [
                "ete3",
                "datetime",
                "statistics",
                "svgwrite",
                "pycairo",
                "cairosvg",
                "argparse",
                "numpy",

                "six"] #could not use ete3 without six

classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Science/Research',
        "Intended Audience :: Healthcare Industry",
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Visualization"]

setup(
    author="Jennifer L Havens",
    author_email='jhavens@ucsd.edu',
    python_requires='>=3.7',
    classifiers=classifiers,
    description="Visualize lineages overtime, with phylogentic context, based on viral genomes",
    packages=find_packages(),#['scripts'],#find_packages(include=['epimuller', 'epimuller.*']),
    entry_points={
        'console_scripts': [
            'epimuller=scripts.mutationLinages_report:main',
            'epimuller-draw=scripts.drawMuller:main',
            'epimuller-define=scripts.defineAndCountClades:main',
            'epimuller-parse=scripts.parseFastaNames:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='epimuller',
    name='epimuller',
    url='https://github.com/jennifer-bio/epimuller',
    version=version,
)
